##############################################################################
# Cognitive Diagnostic Models using Rstan packages
# Code Author: Jorge L. Bazan.
##############################################################################
#Install previously Stand and Rstan


#1. Preliminaries 
rm(list = ls())
set.seed(1234)

#1. Reading data set
# Definition of folder of work
setwd(dirname(rstudioapi::getActiveDocumentContext()$path)) #For R studio

#Defining Q Matrix	
qq <- c(1,1, 
        1,1,
        1,0,
        1,1,
        1,0,
        1,0,
        1,0,
        1,0,
        1,1,
        1,1,
        0,1,
        1,1,
        1,1,
        1,1,
        1,1,
        0,1,
        0,1,
        1,1,
        0,1,
        1,1,
        0,1)
Q <- matrix(qq, ncol=2, byrow=TRUE) # Q-Matrix for our data. Q-Matrix must be defined acordingly to each dataset

#Reading data
bdi=read.csv("BDIdata.csv")

# To test the code in a small sample of the data, uncomment the next line.
#yy=bdi[1:60,4:24]
# To run the code on the full dataset, uncomment the next line.
yy=bdi[,4:24]
Y=as.matrix(yy) #Matrix containing answers, with respondents in each line and items in each column
#edit(Y)



#Running DINA model in CDM package
###################################
if(!require(CDM)) 
  install.packages("CDM")  
library(CDM)

# estimate DINA model
outcdm <- din(Y, Q)
outcdm 
summary(outcdm)

# parameters and standard errors of DINA model
outcdm$item
#probabilities
outcdm$attribute.patt

#Running DINA model in STAN
#############################
#if(!require(rstan)) install.packages("rstan") 
library(rstan)

N <- nrow(Y) #Number of respondents
K <- ncol(Q) #Number of skills
J <- nrow(Q) #Number of items
C <- 2^K #Number of classes

##Visualization of profiles  
delta0 <- rep(1,2^K)
As <- rep(0,K)
for(j in 1:K){
  temp <- combn(1:K,m=j)
  tempmat <- matrix(0,ncol(temp),K)
  for(j in 1:ncol(temp))
    tempmat[j,temp[,j]] <- 1
  As <- rbind(As,tempmat)
}
As <- as.matrix(As)

param <- c("g", "s", "alpha_star", "pi", "u")
datam <- list("N"= N, "J"= J, "K"= K,"C"= C,"Y"= Y, "Q"= Q, "As"= As)

packageVersion("rstan")
print(stan_version())
# Change the value of "Stan.New.Version" to TRUE if stan.version >=2.32

Stan.New.Version = F

if(Stan.New.Version){
    model.file = 'CodeDINAmodel_Stan_2.32.stan'
    }else{
    model.file = 'CodeDINAmodel.stan'
}

options(
    stanc.allow_optimizations = TRUE,
    stanc.auto_format = TRUE
)

stanfit <-  stan(file = 'CodeDINAmodel.stan', 
                pars = param,
                data = datam,
                chains = 1,
                thin = 1,
                iter = 4000,
                warmup = 2000,
                seed = 2)
stanfit

#### Read the g adn s result ####
g <- summary(stanfit, pars="g", probs=NA)$summary
s <- summary(stanfit, pars="s", probs=NA)$summary
pi_stan <- summary(stanfit, pars="pi", probs=NA)$summary
u <- summary(stanfit, pars="u", probs=NA)$summary

#selecting parameters results
coluns <- dim(stanfit)[1]
alpha <- matrix(extract(stanfit, "alpha_star", inc_warmup=FALSE, 
                        permuted=F), ncol=coluns, byrow=TRUE)


#Function to obtain mode of alpha posteriori values 
alpha_moda <- array()
for (i in 1:N) {
  uniqv <- unique(alpha[i,])
  alpha_moda[i] <- uniqv[which.max(tabulate(match(alpha[i,], uniqv)))]
}

resultados <- list("g" =g, "s"= s,"u"= u, "alpha" = alpha_moda,"pi" = pi_stan)
resultados

save.image("CodigoDinaStan.RData")


